@@ -15,6 +15,13 @@ namespace amici {
  */
 class AmiException : public std::exception {
 public:
+    /**
+     * @brief Constructor with printf style interface
+     * @param fmt error message with printf format
+     * @param ... printf formatting variables
+     */
+    AmiException();
+
     /**
      * @brief Constructor with printf style interface
      * @param fmt error message with printf format
@@ -40,6 +47,14 @@ public:
      */
     void storeBacktrace(int nMaxFrames);
 
+  protected:
+    /**
+     * @brief Store the provided message
+     * @param fmt error message with printf format
+     * @param argptr pointer to variadic argument list
+     */
+    void storeMessage(const char *fmt, va_list argptr);
+
 private:
     std::array<char, 500> msg_;
     std::array<char, 500> trace_;
@@ -131,7 +146,13 @@ class IntegrationFailureB : public AmiException  {
  */
 class SetupFailure : public AmiException {
   public:
-    using AmiException::AmiException;
+    /**
+     * @brief Constructor with printf style interface
+     * @param fmt error message with printf format
+     * @param ... printf formatting variables
+     */
+    explicit SetupFailure(char const* fmt, ...);
+
 };
 
 
